/*
 * Google Reader Notifier for Dashboard
 * by RustyBrick.com
 * 
 * Based on Bloglines Reader Notifier
 * by Joe Hughes (joe@retrovirus.com) and Jeremy Koempel (jeremy@smalltransport.com)
 */

var req;
var unread = -1;
var updateFrequency = 5 * 60000; // every 5 minutes (but only when Dashboard is visible)
var user = '';
var pass = '';
var theURL = 'http://www.google.com/reader';
var SID;

var debugEnabled = false;	 
var refreshInterval;

// Utility function for debugging.  When in Dashboard, alerts go to console.

function debug(text) {	 
    if (debugEnabled) {	 
        alert(text);
    }	 
}

// Go to the site--different if you're testing in a browser vs. in Dashboard.

function gotoSite() {
	if (window.widget) {
		widget.openURL(theURL);
	} else {
		window.location.href = theURL;
	}
}

// Creates an instance-specific version of a key for persistent
// Dashboard state storage.

function createkey(key)
{
	return widget.identifier + "-" + key;
}

// If Enter key is pressed on the flip side, flip back to front.

function handleEnter() {
    var keyCode = event.keyCode ? event.keyCode : event.which ? event.which : event.charCode;
    if (keyCode == 13) {
        hidePrefs();
    }
}

// Button states are both in the same image, and are swapped by changing
// the clipping area.  Call pressButton(element ID) when a button is pressed,
// and releaseButton(element ID) when it's released.

function pressButton(id) {
    document.getElementById(id).style.backgroundPosition = '0px -21px'; 
}

function releaseButton(id) {
    document.getElementById(id).style.backgroundPosition = '0px 0px'; 
}

// Perform the flip from front to back.

function showPrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
		widget.prepareForTransition("ToBack");		// freezes the widget so that you can change it without the user noticing
	
	front.style.display="none";		// hide the front
	back.style.display="block";		// show the back
	
	txt = document.getElementById("userField");
    txt.focus();  // so you can start typing right away
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);	// and flip the widget over	
}

// Perform the flip from back to front, refresh content under the assumption
// that the configuration has changed.

function hidePrefs()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
	
	back.style.display="none";			// hide the back
	front.style.display="block";		// show the front
	
	txt = document.getElementById("userField");
	user = txt.value;
	
	txt = document.getElementById("passField");
	pass = txt.value;
	
	if (window.widget) {
        widget.setPreferenceForKey(user, createkey("user")); // save value
        widget.setPreferenceForKey(pass, createkey("pass")); // save value
    }
	SID = "";
	
	unread = -1;
	refreshCount();
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front
}

// Called when the XMLHttpRequest status changes--if it's got a successful
// response, we update the display with the new value.

function processReqChange() {
    if ((req.readyState == 4) && (req.status == 200)) {    	
    	var obj = eval('(' + req.responseText + ')');

    	var count = 0;
    	
    	for (x=0; x<obj.unreadcounts.length; x++) {
    	
    		// need to ignore folders
    		
    		if (obj.unreadcounts[x].id.substring(0,5) == 'feed/') {
    			count += parseInt(obj.unreadcounts[x].count);
    		}
    	}
    	
        unread = count;
        
        updateDisplay();
    }
}

function parseSID(text)
{
	var ret = text.split('=')[1];
	return ret.substring(0,ret.length-5);
}

// Start the process of updating the unread count.

function refreshCount() {
    if (window.widget) {	 
        debug('refreshing content')	 
    }
    
    // ignore auth if already auth'd

    if (SID != undefined && SID.length > 0) {
    	updateCount2();
    	return;
    }
    
    if (!user || (user.length == 0)) {
        if (window.widget) {
            debug('no user, not refreshing');
            return;
        }
    }
    
    if (!pass || (pass.length == 0)) {
        if (window.widget) {
            debug('no pass, not refreshing');
            return;
        }
    }

    var params = "service=reader" +
            "&Email="    + escape(user) +
            "&Passwd="   + escape(pass) +
            "&source="   + escape("test") +
            "&continue=" + escape("http://www.google.com/");
    
    req = new XMLHttpRequest();
    req.open("POST", "https://www.google.com/accounts/ClientLogin", true);
    req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
	req.setRequestHeader("Content-length", params.length);
	req.setRequestHeader("Connection", "close");
	req.onreadystatechange = function() {//Call a function when the state changes.
		if(req.readyState == 4 && req.status == 200) {
			SID = parseSID(req.responseText);
			updateCount2();
		}
		else {
			count = -1;
			updateDisplay();
		}
	}
	req.send(params);
}

function updateCount2()
{
	var params =
			"?all=true&output=json" +
			"&ck="     + escape((new Date).getTime()) +
			"&client=" + "test";
	
	req = new XMLHttpRequest();
	req.open("GET", "http://www.google.com/reader/api/0/unread-count" + params, true);
	req.setRequestHeader("Cookie", "SID="+SID);
	req.onreadystatechange = processReqChange;
	req.send(null);
}

// Update the HTML content for the unread count based on the unread variable.

function updateDisplay() {
    if (window.widget) {	 
        debug('updating display to reflect ' + unread + ' unread');	 
    }
    
    countdiv = document.getElementById("count");
    
    var unreadtext = "" + unread;
    if (unread < 0) {
        unreadtext = "error";
    }
    countdiv.innerHTML = unreadtext;
}

// We don't want the widget refreshing when it's not on the screen, so these
// functions start and stop the refresh timer.

function setRefreshTimer() {	 
    if (refreshInterval == null) {	 
        debug("setting refresh timer");	 
        refreshInterval = setInterval("refreshCount();", updateFrequency);	 
    }	 
}	 
 	 
function clearRefreshTimer() {	 
    if (refreshInterval != null) {	 
        debug("clearing refresh timer");	 
        clearInterval(refreshInterval);	 
        refreshInterval = null;
    }
}
 
function isEmailValid() {
    return (user && (user.length > 0) &&
    		pass && (pass.length > 0));
}

              
function init() {
    // Attempt to retrieve saved configuration.
    if (window.widget) {
        user = widget.preferenceForKey(createkey("user"));
        if (!user || (user == "undefined")) {
            user = "";
        }
        
        pass = widget.preferenceForKey(createkey("pass"));
        if (!pass || (pass == "undefined")) {
            pass = "";
        }
    }
    
	txt = document.getElementById("userField");
	txt.value = user;

	txt = document.getElementById("passField");
	txt.value = pass;
	
    if (window.widget) {	 
        widget.onshow = function OnShow() { // when Dashboard triggered	 
            debug("widget shown");	 
            setRefreshTimer();	 
            refreshCount();	 
        }	 
        widget.onremove = function OnRemove() { // when widget discarded 
            debug("widget removed");
            widget.setPreferenceForKey(null, createkey("user"));	 
            widget.setPreferenceForKey(null, createkey("pass"));	 
            clearRefreshTimer();	 
        }	 
        widget.onhide = function OnHide() { // when Dashboard dismissed	 
            debug("widget hidden");	 
            clearRefreshTimer();	 
        }	 
    }	 
 	
    setRefreshTimer();
    
    if (!isEmailValid()) { // go directly to config if necessary
        setTimeout ('showPrefs();', 1000); // don't interfere with ripples
    } else {
        refreshCount();
    }	 
}
